#pragma once

#include <QObject>

class QWidget;
class QWindow;

class UkuiWindowHelper : public QObject
{
    Q_OBJECT
public:
    explicit UkuiWindowHelper(QWidget *widget);
    explicit UkuiWindowHelper(QWindow *window);
    ~UkuiWindowHelper();

    enum class WindowRole {
        Normal,
        Desktop,
        Panel,
        OnScreenDisplay,
        Notification,
        Tooltip,
        CriticalNotification,
        AppletPopup,
        ScreenLock,
        Watermark,
        SystemWindow,
        InputPanel,
        Logout,
        ScreenLockNotification,
        Switcher,
        Authentication
    };
    Q_ENUM(WindowRole)

    enum class State : uint32_t {
        Minimizable = 0x1,
        Maximizable = 0x2,
        Closeable = 0x4,
        Fullscreenable = 0x8,
        Movable = 0x10,
        Resizable = 0x20,
        Focusable = 0x40,
        Activatable = 0x80,
        KeepAbove = 0x100,
        KeepBelow = 0x200,
    };
    Q_DECLARE_FLAGS(States, State)

    enum class Position {
        Top,
        Bottom,
        Left,
        Right
    };
    Q_ENUM(Position)

    void removeTitleBar();
    void setWindowRole(WindowRole role);
    void setWindowState(States state);
    void setWindowIcon(const QString &iconName);
    void setSkipTaskBar(bool skip);
    void setSkipSwitcher(bool skip);
    void setPanelAutoHide(bool autoHide);
    void grabKeyboard();
    void openUnderCursor(QPoint pos);
    void setBlurEffect(QRegion region, int level = 0, bool enable = true);
    void setSlideEffect(Position position, int offset = 0, bool enable = true);
    States defaultState() const;

    friend inline State operator&(State lhs, State rhs)
    {
        return static_cast<State>(static_cast<uint32_t>(lhs) & static_cast<uint32_t>(rhs));
    }

    friend inline State operator~(State rhs)
    {
        return static_cast<State>(~static_cast<uint32_t>(rhs));
    }

private:
    QWidget *m_widget = nullptr;
    QWindow *m_window = nullptr;
};
